@@ -22,7 +22,7 @@ from parser import DirectoryParser
 from timezones import PacificTimezone
 from utils import urljoin
 
-import progressbar
+import progressbar as pb
 
 version = pkg_resources.require("mozdownload")[0].version
 
@@ -33,7 +33,7 @@ applications.
 mozdownload version: %(version)s
 """ % {'version': version}
 
-APPLICATIONS = ['b2g', 'firefox', 'thunderbird']
+APPLICATIONS = ('b2g', 'firefox', 'thunderbird')
 
 # Base URL for the path to all builds
 BASE_URL = 'https://ftp.mozilla.org/pub/mozilla.org'
@@ -44,7 +44,7 @@ CHUNK_SIZE = 16 * 1024
 PLATFORM_FRAGMENTS = {'linux': 'linux-i686',
                       'linux64': 'linux-x86_64',
                       'mac': 'mac',
-                      'mac64': 'mac64',
+                      'mac64': r'mac(64)?',
                       'win32': 'win32',
                       'win64': 'win64-x86_64'}
 
@@ -55,6 +55,14 @@ DEFAULT_FILE_EXTENSIONS = {'linux': 'tar.bz2',
                            'win32': 'exe',
                            'win64': 'exe'}
 
+MULTI_LOCALE_APPLICATIONS = ('b2g')
+
+
+class NotSupportedError(Exception):
+    """Exception for a build not being supported"""
+    def __init__(self, message):
+        Exception.__init__(self, message)
+
 
 class NotFoundError(Exception):
     """Exception for a resource not being found (e.g. no logs)"""
@@ -80,7 +88,7 @@ class Scraper(object):
     """Generic class to download an application from the Mozilla server"""
 
     def __init__(self, directory, version, platform=None,
-                 application='firefox', locale='en-US', extension=None,
+                 application='firefox', locale=None, extension=None,
                  authentication=None, retry_attempts=0, retry_delay=10.,
                  is_stub_installer=False, timeout=None, log_level='INFO',
                  base_url=BASE_URL):
@@ -90,10 +98,16 @@ class Scraper(object):
         self._binary = None
 
         self.directory = directory
-        self.locale = locale
+        if not locale:
+            if application in MULTI_LOCALE_APPLICATIONS:
+                self.locale = 'multi'
+            else:
+                self.locale = 'en-US'
+        else:
+            self.locale = locale
+
         self.platform = platform or self.detect_platform()
         self.version = version
-        self.extension = extension or DEFAULT_FILE_EXTENSIONS[self.platform]
         self.authentication = authentication
         self.retry_attempts = retry_attempts
         self.retry_delay = retry_delay
@@ -108,6 +122,16 @@ class Scraper(object):
         self.application = application
         self.base_url = urljoin(base_url, self.application)
 
+        if extension:
+            self.extension = extension
+        else:
+            if self.application in MULTI_LOCALE_APPLICATIONS and \
+                    self.platform in ('win32', 'win64'):
+                # builds for MULTI_LOCALE_APPLICATIONS only exist in zip
+                self.extension = 'zip'
+            else:
+                self.extension = DEFAULT_FILE_EXTENSIONS[self.platform]
+
         attempt = 0
         while True:
             attempt += 1
@@ -275,25 +299,27 @@ class Scraper(object):
                                  auth=self.authentication)
                 r.raise_for_status()
 
+                content_length = r.headers.get('Content-length')
                 # ValueError: Value out of range if only total_size given
-                total_size = int(r.headers.get('Content-length').strip())
-                max_value = ((total_size / CHUNK_SIZE) + 1) * CHUNK_SIZE
+                if content_length:
+                    total_size = int(content_length.strip())
+                    max_value = ((total_size / CHUNK_SIZE) + 1) * CHUNK_SIZE
+
                 bytes_downloaded = 0
 
                 log_level = self.logger.getEffectiveLevel()
-                if log_level <= mozlog.INFO:
-                    widgets = [progressbar.Percentage(), ' ',
-                               progressbar.Bar(), ' ', progressbar.ETA(),
-                               ' ', progressbar.FileTransferSpeed()]
-                    pbar = progressbar.ProgressBar(widgets=widgets,
-                                                   maxval=max_value).start()
+                if log_level <= mozlog.INFO and content_length:
+                    widgets = [pb.Percentage(), ' ', pb.Bar(), ' ', pb.ETA(),
+                               ' ', pb.FileTransferSpeed()]
+                    pbar = pb.ProgressBar(widgets=widgets,
+                                          maxval=max_value).start()
 
                 with open(tmp_file, 'wb') as f:
                     for chunk in iter(lambda: r.raw.read(CHUNK_SIZE), ''):
                         f.write(chunk)
                         bytes_downloaded += CHUNK_SIZE
 
-                        if log_level <= mozlog.INFO:
+                        if log_level <= mozlog.INFO and content_length:
                             pbar.update(bytes_downloaded)
 
                         t1 = total_seconds(datetime.now() - start_time)
@@ -301,7 +327,7 @@ class Scraper(object):
                                 t1 >= self.timeout_download:
                             raise TimeoutError
 
-                if log_level <= mozlog.INFO:
+                if log_level <= mozlog.INFO and content_length:
                     pbar.finish()
                 break
             except (requests.exceptions.RequestException, TimeoutError), e:
@@ -309,7 +335,7 @@ class Scraper(object):
                     os.remove(tmp_file)
                 if self.retry_attempts > 0:
                     # Log only if multiple attempts are requested
-                    self.logger.warning('Download failed: "%s"' % e.message)
+                    self.logger.warning('Download failed: "%s"' % str(e))
                     self.logger.info('Will retry in %s seconds...' %
                                      (self.retry_delay))
                     time.sleep(self.retry_delay)
@@ -361,8 +387,10 @@ class DailyScraper(Scraper):
         elif self.date:
             # A date (without time) has been specified. Use its value and the
             # build index to find the requested build for that day.
-            self.date = datetime.strptime(self.date, '%Y-%m-%d')
-
+            try:
+                self.date = datetime.strptime(self.date, '%Y-%m-%d')
+            except:
+                raise ValueError('%s is not a valid date' % self.date)
         else:
             # If no build id nor date have been specified the latest available
             # build of the given branch has to be identified. We also have to
@@ -398,6 +426,11 @@ class DailyScraper(Scraper):
         """Return whether or not the given dir contains a build."""
 
         url = urljoin(self.base_url, self.monthly_build_list_regex, dir)
+
+        if self.application in MULTI_LOCALE_APPLICATIONS \
+                and self.locale != 'multi':
+            url = urljoin(url, self.locale)
+
         parser = DirectoryParser(url, authentication=self.authentication,
                                  timeout=self.timeout_network)
 
@@ -421,7 +454,8 @@ class DailyScraper(Scraper):
         regex = r'%(DATE)s-(\d+-)+%(BRANCH)s%(L10N)s$' % {
             'DATE': date.strftime('%Y-%m-%d'),
             'BRANCH': self.branch,
-            'L10N': '' if self.locale == 'en-US' else '(-l10n)?'}
+            # ensure to select the correct subfolder for localized builds
+            'L10N': '' if self.locale in ('en-US', 'multi') else '(-l10n)?'}
         parser.entries = parser.filter(regex)
         parser.entries = parser.filter(self.is_build_dir)
 
@@ -441,6 +475,7 @@ class DailyScraper(Scraper):
         self.show_matching_builds(parser.entries)
         if build_index is None:
             build_index = len(parser.entries) - 1
+        self.logger.info('Selected build: %s' % parser.entries[build_index])
 
         return (parser.entries, build_index)
 
@@ -495,6 +530,9 @@ class DailyScraper(Scraper):
         try:
             path = urljoin(self.monthly_build_list_regex,
                            self.builds[self.build_index])
+            if self.application in MULTI_LOCALE_APPLICATIONS \
+                    and self.locale != 'multi':
+                path = urljoin(path, self.locale)
             return path
         except:
             folder = urljoin(self.base_url, self.monthly_build_list_regex)
@@ -579,9 +617,13 @@ class ReleaseCandidateScraper(ReleaseScraper):
         # Internally we access builds via index
         self.builds, self.build_index = self.get_build_info_for_version(
             self.version)
-        if self.build_number is not None:
+        if self.build_number and \
+                ('build%s' % self.build_number) in self.builds:
             self.builds = ['build%s' % self.build_number]
             self.build_index = 0
+            self.logger.info('Selected build: build%s' % self.build_number)
+        else:
+            self.logger.info('Selected build: build%s' % self.build_index)
 
     def get_build_info_for_version(self, version, build_index=None):
         url = urljoin(self.base_url, self.candidate_build_list_regex)
@@ -609,7 +651,7 @@ class ReleaseCandidateScraper(ReleaseScraper):
            a candidate build."""
 
         # Regex for possible builds for the given date
-        return r'nightly/%(VERSION)s-candidates/' % {
+        return r'candidates/%(VERSION)s-candidates/' % {
             'VERSION': self.version}
 
     @property
@@ -692,8 +734,12 @@ class TinderboxScraper(Scraper):
                 # date is provided in the format 2013-07-23
                 self.date = datetime.strptime(self.date, '%Y-%m-%d')
             except:
-                # date is provided as a unix timestamp
-                self.timestamp = self.date
+                try:
+                    # date is provided as a unix timestamp
+                    datetime.fromtimestamp(float(self.date))
+                    self.timestamp = self.date
+                except:
+                    raise ValueError('%s is not a valid date' % self.date)
 
         self.locale_build = self.locale != 'en-US'
         # For localized builds we do not have to retrieve the list of builds
@@ -797,6 +843,7 @@ class TinderboxScraper(Scraper):
         # If no index has been given, set it to the last build of the day.
         if build_index is None:
             build_index = len(parser.entries) - 1
+        self.logger.info('Selected build: %s' % parser.entries[build_index])
 
         return (parser.entries, build_index)
 
@@ -854,9 +901,9 @@ def cli():
                            'and tinderbox builds)')
     parser.add_option('--locale', '-l',
                       dest='locale',
-                      default='en-US',
                       metavar='LOCALE',
-                      help='Locale of the application, default: "%default"')
+                      help='Locale of the application, default: "en-US or '
+                           'multi"')
     parser.add_option('--platform', '-p',
                       dest='platform',
                       choices=PLATFORM_FRAGMENTS.keys(),
@@ -975,7 +1022,7 @@ def cli():
     # Check for required options and arguments
     # Note: Will be optional when ini file support has been landed
     if not options.url \
-       and not options.type in ['daily', 'tinderbox'] \
+       and options.type not in ['daily', 'tinderbox'] \
        and not options.version:
         parser.error('The version of the application to download has not'
                      ' been specified.')
@@ -1010,6 +1057,10 @@ def cli():
     kwargs = scraper_keywords.copy()
     kwargs.update(scraper_options.get(options.type, {}))
 
+    if options.application == 'b2g' and \
+            options.type in ('candidate', 'release'):
+        error_msg = "%s build is not yet supported for B2G" % options.type
+        raise NotSupportedError(error_msg)
     if options.url:
         build = DirectScraper(options.url, **kwargs)
     else:
